@@ -29,12 +29,11 @@ import json
 
 from uuid import UUID
 from dns import resolver
-from ldap3 import ALL_ATTRIBUTES, BASE
-from ldap3.utils.config import _ATTRIBUTES_EXCLUDED_FROM_CHECK
+from ldap3 import ALL_ATTRIBUTES, BASE, SUBTREE, LEVEL
 from ldap3.core.exceptions import LDAPKeyError, LDAPAttributeError, LDAPCursorError, LDAPNoSuchObjectResult, LDAPSocketReceiveError, LDAPSocketSendError
 from ldap3.protocol.microsoft import security_descriptor_control
 # from impacket.krb5.kerberosv5 import KerberosError
-from bloodhound.ad.utils import ADUtils, DNSCache, SidCache, SamCache
+from bloodhound.ad.utils import ADUtils, DNSCache, SidCache, SamCache, CollectionException
 from bloodhound.ad.computer import ADComputer
 from bloodhound.enumeration.objectresolver import ObjectResolver
 from future.utils import itervalues, iteritems, native_str
@@ -67,7 +66,7 @@ class ADDC(ADComputer):
         for r in q:
             ip = r.address
 
-        ldap = self.ad.auth.getLDAPConnection(hostname=ip,
+        ldap = self.ad.auth.getLDAPConnection(hostname=self.hostname, ip=ip,
                                               baseDN=self.ad.baseDN, protocol=protocol)
         if resolver:
             self.resolverldap = ldap
@@ -113,15 +112,17 @@ class ADDC(ADComputer):
                 except (resolver.NXDOMAIN, resolver.Timeout):
                     continue
 
-        self.gcldap = self.ad.auth.getLDAPConnection(hostname=ip, gc=True,
+        self.gcldap = self.ad.auth.getLDAPConnection(hostname=self.hostname, ip=ip, gc=True,
                                                      baseDN=self.ad.baseDN, protocol=protocol)
         return self.gcldap is not None
 
-    def search(self, search_filter='(objectClass=*)', attributes=None, search_base=None, generator=True, use_gc=False, use_resolver=False, query_sd=False, is_retry=False):
+    def search(self, search_filter='(objectClass=*)',attributes=None, search_base=None, generator=True, use_gc=False, use_resolver=False, query_sd=False, is_retry=False,  search_scope=SUBTREE,):
         """
         Search for objects in LDAP or Global Catalog LDAP.
         """
-        if self.ldap is None:
+        if self.ldap is None and not use_resolver:
+            self.ldap_connect(resolver=use_resolver)
+        if self.resolverldap is None and use_resolver:
             self.ldap_connect(resolver=use_resolver)
         if search_base is None:
             search_base = self.ad.baseDN
@@ -152,6 +153,7 @@ class ADDC(ADComputer):
                                                         search_filter,
                                                         attributes=attributes,
                                                         paged_size=200,
+                                                        search_scope=search_scope,
                                                         controls=controls,
                                                         generator=generator)
         try:
@@ -280,6 +282,14 @@ class ADDC(ADComputer):
             except IndexError:
                 pass
 
+        if entriesNum == 0:
+            # Raise exception if we somehow managed to authenticate but the domain is wrong
+            # prevents confusing exceptions later
+            actualdn = self.ldap.server.info.other['defaultNamingContext'][0]
+            actualdomain = ADUtils.ldap2domain(actualdn)
+            logging.error('Could not find the requested domain %s on this DC, LDAP server reports is domain as %s (you may want to try that?)', self.ad.domain, actualdomain)
+            raise CollectionException("Specified domain was not found in LDAP")
+
         logging.info('Found %u domains', entriesNum)
 
         return entries
@@ -358,6 +368,42 @@ class ADDC(ADComputer):
                               query_sd=acl)
         return entries
 
+    def get_gpos(self, include_properties=False, acl=False):
+        properties = ['distinguishedName', 'name', 'objectGUID', 'gPCFileSysPath', 'displayName']
+        if include_properties:
+            properties += ['description', 'whencreated']
+        if acl:
+            properties += ['nTSecurityDescriptor']
+        entries = self.search('(objectCategory=groupPolicyContainer)',
+                              properties,
+                              generator=True,
+                              query_sd=acl)
+        return entries
+
+    def get_ous(self, include_properties=False, acl=False):
+        properties = ['distinguishedName', 'name', 'objectGUID', 'gPLink']
+        if include_properties:
+            properties += ['description', 'whencreated']
+        if acl:
+            properties += ['nTSecurityDescriptor']
+        entries = self.search('(objectCategory=organizationalUnit)',
+                              properties,
+                              generator=True,
+                              query_sd=acl)
+        return entries
+
+    def get_containers(self, include_properties=False, acl=False, dn=''):
+        properties = ['distinguishedName', 'name', 'objectGUID', 'isCriticalSystemObject','objectClass', 'objectCategory']
+        if include_properties:
+            properties += ['description', 'whencreated']
+        if acl:
+            properties += ['nTSecurityDescriptor']
+        entries = self.search('(&(objectCategory=container)(objectClass=container))',
+                              properties,
+                              generator=True,
+                              query_sd=acl,
+                              search_base=dn)
+        return entries
 
     def get_users(self, include_properties=False, acl=False):
 
@@ -412,7 +458,7 @@ class ADDC(ADComputer):
                 properties += ['nTSecurityDescriptor', 'ms-mcs-admpwdexpirationtime']
             else:
                 properties.append('nTSecurityDescriptor')
-        entries = self.search('(&(sAMAccountType=805306369)(!(UserAccountControl:1.2.840.113556.1.4.803:=2)))',
+        entries = self.search('(&(sAMAccountType=805306369))',
                               properties,
                               generator=True,
                               query_sd=acl)
@@ -458,6 +504,15 @@ class ADDC(ADComputer):
                               ['homedirectory', 'scriptpath', 'profilepath'])
         return entries
 
+    def get_childobjects(self, dn, use_resolver=True):
+        entries = self.search('(|(objectClass=container)(objectClass=organizationalUnit)(sAMAccountType=805306369)(objectClass=group)(&(objectCategory=person)(objectClass=user)))',
+                              attributes=['objectSid', 'objectClass', 'objectGUID', 'distinguishedName', 'sAMAccountName', 'sAMAccountType'],
+                              search_base=dn,
+                              search_scope=LEVEL,
+                              use_resolver=use_resolver)
+                              
+        return entries
+
     def get_trusts(self):
         entries = self.search('(objectClass=trustedDomain)',
                               attributes=['flatName', 'name', 'securityIdentifier', 'trustAttributes', 'trustDirection', 'trustType'],
@@ -571,7 +626,7 @@ class AD(object):
     def save_cachefile(self, cachefile):
         pass
 
-    def dns_resolve(self, domain=None, kerberos=True, options=None):
+    def dns_resolve(self, domain=None, options=None):
         logging.debug('Querying domain controller information from DNS')
 
         basequery = '_ldap._tcp.pdc._msdcs'
@@ -624,17 +679,27 @@ class AD(object):
                 else:
                     logging.warning('Could not find a global catalog server. Please specify one with -gc')
 
-        if kerberos is True:
-            try:
-                q = self.dnsresolver.query('_kerberos._tcp.dc._msdcs', 'SRV', tcp=self.dns_tcp)
-                for r in q:
-                    kdc = str(r.target).rstrip('.')
-                    logging.debug('Found KDC: %s' % str(r.target).rstrip('.'))
-                    if kdc not in self._kdcs:
-                        self._kdcs.append(kdc)
-                        self.auth.kdc = self._kdcs[0]
-            except resolver.NXDOMAIN:
-                pass
+        try:
+            kquery = query.replace('pdc','dc').replace('_ldap','_kerberos')
+            q = self.dnsresolver.query(kquery, 'SRV', tcp=self.dns_tcp)
+            # TODO: Get the additional records here to get the DC ip immediately
+            for r in q:
+                kdc = str(r.target).rstrip('.')
+                logging.debug('Found KDC for enumeration domain: %s' % str(r.target).rstrip('.'))
+                if kdc not in self._kdcs:
+                    self._kdcs.append(kdc)
+                    self.auth.kdc = self._kdcs[0]
+        except resolver.NXDOMAIN:
+            pass
+
+        if self.auth.userdomain.lower() != ad_domain.lower():
+            # Resolve KDC for user auth domain
+            kquery = '_kerberos._tcp.dc._msdcs.%s' % self.auth.userdomain
+            q = self.dnsresolver.query(kquery, 'SRV', tcp=self.dns_tcp)
+            for r in q:
+                kdc = str(r.target).rstrip('.')
+                logging.debug('Found KDC for user: %s' % str(r.target).rstrip('.'))
+                self.auth.userdomain_kdc = kdc
 
         return True
 
@@ -650,6 +715,23 @@ class AD(object):
                 return entry
         return None
 
+
+    def get_dn_from_cache_or_ldap(self, distinguishedname):
+        try:
+            linkentry = self.dncache[distinguishedname.upper()]
+        except KeyError:
+            use_gc = ADUtils.ldap2domain(distinguishedname).lower() != self.domain.lower()
+            qobject = self.objectresolver.resolve_distinguishedname(distinguishedname, use_gc=use_gc)
+            if qobject is None:
+                return None
+            resolved_entry = ADUtils.resolve_ad_entry(qobject)
+            linkentry = {
+                "ObjectIdentifier": resolved_entry['objectid'],
+                "ObjectType": resolved_entry['type'].capitalize()
+            }
+            self.dncache[distinguishedname.upper()] = linkentry
+        return linkentry
+
 """
 Active Directory Domain
 """
