@@ -29,6 +29,8 @@ import re
 import dns
 from dns import resolver, reversename
 from bloodhound.ad.structures import LDAP_SID
+from bloodhound.ad.dumpntlm import DumpNtlm
+from impacket.dcerpc.v5 import transport, wkst
 
 """
 """
@@ -156,6 +158,82 @@ class ADUtils(object):
 
         return result
 
+    @staticmethod
+    def rpc_get_hostname(ip, adauth):
+        '''
+        Authenticated hostname resolution. Requires admin rights.
+        Currently not in use - does not respect authentication stack
+        '''
+        result = ip
+
+        binding = r'ncacn_np:%s[\PIPE\wkssvc]' % ip
+        rpctransportWkst = transport.DCERPCTransportFactory(binding)
+        if hasattr(rpctransportWkst, 'set_credentials'):
+            rpctransportWkst.set_credentials(adauth.username, adauth.password, 
+                                adauth.domain, adauth.lm_hash, adauth.nt_hash, adauth.aes_key)
+            rpctransportWkst.set_kerberos(False, adauth.kdc) # not supported yet
+        dce = rpctransportWkst.get_dce_rpc()
+
+        try:
+            dce.connect()
+            dce.bind(wkst.MSRPC_UUID_WKST)
+            resp = wkst.hNetrWkstaGetInfo(dce, 100)
+
+            result = resp['WkstaInfo']['WkstaInfo100']['wki100_computername'][:-1]
+            domain_name = resp['WkstaInfo']['WkstaInfo100']['wki100_langroup'][:-1]
+            if '.' in domain_name: # not workgroup
+                result = '.'.join([result, domain_name])
+
+            dce.disconnect()
+        except:
+            logging.warning('WkstaGetInfo failed: %s' % ip)
+            pass
+
+        return result
+
+    @staticmethod
+    def get_ntlm_hostname(ip):
+        '''
+        Unauthenticated hostname resolution with NTLM challenge/response
+        '''
+        def parse_info(info):
+            if not info:
+                return info
+            if 'dns_host' in info:
+                return info['dns_host'].lower()
+            elif 'name' in info:
+                result = info['name']
+                if 'dns_domain' in info and '.' in info['dns_domain']:
+                    result = '.'.join((result, info['dns_domain']))
+                return result.lower()
+            return None
+        
+        try:
+            ntlm_dumper = DumpNtlm(ip, ip, 445)
+            hostname = parse_info(ntlm_dumper.GetInfo())
+            if hostname:
+                return hostname
+        except:
+            pass
+
+        try:
+            ntlm_dumper = DumpNtlm(ip, ip, 139)
+            hostname = parse_info(ntlm_dumper.GetInfo())
+            if hostname:
+                return hostname
+        except:
+            pass
+
+        try:
+            ntlm_dumper = DumpNtlm(ip, ip, 135)
+            hostname = parse_info(ntlm_dumper.GetInfo())
+            if hostname:
+                return hostname
+        except:
+            pass
+
+        return ip
+
     # Translate the binary SID from LDAP into human-readable form
     @staticmethod
     def formatSid(siddata):
@@ -179,6 +257,14 @@ class ADUtils(object):
         # must not be an empty string since this breaks our CSV files
         return 'Unknown'
 
+    @staticmethod
+    def is_dc(entry):
+        """
+        Check whether the computer object is a Domain Controller
+        """
+        uac = ADUtils.get_entry_property(entry, 'userAccountControl', default=0)
+        return uac & 0x00002000 == 0x00002000
+
     @staticmethod
     def resolve_ad_entry(entry):
         """
@@ -196,7 +282,7 @@ class ADUtils(object):
         resolved['objectid'] = ADUtils.get_entry_property(entry, 'objectSid', '')
         resolved['principal'] = ('%s@%s' % (account, domain)).upper()
         if not ADUtils.get_entry_property(entry, 'sAMAccountName'):
-            if 'ForeignSecurityPrincipals' in dn:
+            if 'ForeignSecurityPrincipals' in dn and not 'container' in ADUtils.get_entry_property(entry, 'objectClass', []):
                 resolved['principal'] = domain.upper()
                 resolved['type'] = 'foreignsecurityprincipal'
                 ename = ADUtils.get_entry_property(entry, 'name')
@@ -211,6 +297,16 @@ class ADUtils(object):
                     else:
                         # Foreign security principal
                         resolved['objectid'] = ename
+            elif ADUtils.get_entry_property(entry, 'objectGUID'):
+                resolved['objectid'] = ADUtils.get_entry_property(entry, 'objectGUID', '').upper()[1:-1]
+                resolved['principal'] = ('%s@%s' % (ADUtils.get_entry_property(entry, 'name', ''), domain)).upper()
+                object_class = ADUtils.get_entry_property(entry, 'objectClass', [])
+                if 'organizationalUnit' in object_class:
+                    resolved['type'] = 'OU'
+                elif 'container' in object_class:
+                    resolved['type'] = 'Container'
+                else:
+                    resolved['type'] = 'Base'
             else:
                 resolved['type'] = 'Base'
         else:
@@ -318,6 +414,29 @@ class ADUtils(object):
             data = repr(data)
         return data
 
+    @staticmethod
+    def is_filtered_container(containerdn):
+        if "CN=DOMAINUPDATES,CN=SYSTEM,DC=" in containerdn.upper():
+            return True
+        if "CN=POLICIES,CN=SYSTEM,DC=" in containerdn.upper() and (containerdn.upper().startswith('CN=USER') or containerdn.upper().startswith('CN=MACHINE')):
+            return True
+        return False
+
+    @staticmethod
+    def is_filtered_container_child(containerdn):
+        if "CN=PROGRAM DATA,DC=" in containerdn.upper():
+            return True
+        if "CN=SYSTEM,DC=" in containerdn.upper():
+            return True
+        return False
+
+    @staticmethod
+    def parse_gplink_string(linkstr):
+        if not linkstr:
+            return
+        for links in linkstr.split('[LDAP://')[1:]:
+            dn, options = links.rstrip('][').split(';')
+            yield dn, int(options)
 
 class AceResolver(object):
     """
@@ -462,3 +581,6 @@ class SamCache(SidCache):
     Identical to the SidCache in behaviour
     """
     pass
+
+class CollectionException(Exception):
+    pass
