@@ -23,15 +23,10 @@
 ####################
 from __future__ import unicode_literals
 import logging
-import threading
 from multiprocessing import Pool
-from ldap3.utils.conv import escape_filter_chars
 from impacket.uuid import string_to_bin, bin_to_string
-from bloodhound.ad.utils import ADUtils
 from bloodhound.lib import cstruct
 from io import BytesIO
-import binascii
-import pprint
 from future.utils import iteritems, native_str
 
 # Extended rights and property GUID mapping, converted to binary so we don't have to do this
@@ -137,7 +132,7 @@ def parse_binary_acl(entry, entrytype, acl, objecttype_guid_map):
             writeprivs = ace_object.acedata.mask.has_priv(ACCESS_MASK.ADS_RIGHT_DS_WRITE_PROP)
             if writeprivs:
                 # GenericWrite
-                if entrytype in ['user', 'group', 'computer'] and not ace_object.acedata.has_flag(ACCESS_ALLOWED_OBJECT_ACE.ACE_OBJECT_TYPE_PRESENT):
+                if entrytype in ['user', 'group', 'computer', 'gpo'] and not ace_object.acedata.has_flag(ACCESS_ALLOWED_OBJECT_ACE.ACE_OBJECT_TYPE_PRESENT):
                     relations.append(build_relation(sid, 'GenericWrite', inherited=is_inherited))
                 if entrytype == 'group' and can_write_property(ace_object, EXTRIGHTS_GUID_MAPPING['WriteMember']):
                     relations.append(build_relation(sid, 'AddMember', '', inherited=is_inherited))
@@ -201,7 +196,8 @@ def parse_binary_acl(entry, entrytype, acl, objecttype_guid_map):
 
             if mask.has_priv(ACCESS_MASK.ADS_RIGHT_DS_WRITE_PROP):
                 # Genericwrite is only for properties, don't skip after
-                relations.append(build_relation(sid, 'GenericWrite', inherited=is_inherited))
+                if entrytype in ['user', 'group', 'computer', 'gpo']:
+                    relations.append(build_relation(sid, 'GenericWrite', inherited=is_inherited))
 
             if mask.has_priv(ACCESS_MASK.WRITE_OWNER):
                 relations.append(build_relation(sid, 'WriteOwner', inherited=is_inherited))
